#%% #!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Preprocessing of the images

1. Compute maximum projections.
2. Explore the intensities of the maximum projection images.
3. Correct the intensities. Normalisation across channels and images, 
to equalise the standard deviation for intensity values > th (just considering 
the pixels that contain protein, and not those represinting air, so as to avoid
enhancing the background noise)

Created on Wed Oct  7 10:34:52 2020

@author: monj
"""
import os
import microscopy_analysis as ma
import matplotlib.pyplot as plt
from datetime import datetime
import numpy as np
import skimage.io as io
import copy

# Turn interactive plotting off
plt.ioff()

plt.close('all')

start_time_global = datetime.now()

#%% I/O directories

diseases = ['emphysema','uip','sarcoidosis','covid'] #I don't include sarcoidosis and uip, I manually create the group later, just so those don't count twice for the std estimation
conditions = diseases[:]
conditions.insert(0,'control')

# Input directories - images are stored in folders starting with the name 'frame'
base_dir = '../input_data/'
dir_in = base_dir + '/raw/'
base_name = 'frame'

# Output directories: max projections and postprocessed
dir_maxProj_raw = base_dir + 'maximum_projections/' 
ma.make_output_dirs(dir_maxProj_raw,conditions)

dir_postprocessed = base_dir + 'postprocessed_maxProjs/'

dir_maxProj_processed = dir_postprocessed + 'rgb/' 
ma.make_output_dirs(dir_maxProj_processed,conditions)
    
dir_maxProj_processed_cmy = dir_postprocessed + 'cmy/' 
ma.make_output_dirs(dir_maxProj_processed_cmy,conditions)

dir_results_intinspect = dir_postprocessed + 'intensity_inspection/' 
ma.make_output_dirs(dir_results_intinspect,conditions)

#%% Get maximum projection images

# Automatically decide whether to recompute or read the max. projections images
read_flag = True
for disease in diseases:
    disease_flag = len(os.listdir(dir_maxProj_raw + disease + '/'))>2
    #print(disease)
    #print(disease_flag)
    read_flag = disease_flag&read_flag
     

# Get maximum projection images
if not(read_flag):
    # Compute maximum projection images (runtime ~= 135 s)
    print('Computing maximum projection images')
    startTime = datetime.now()
    max_img_list = []
    for condition in conditions:
        max_img_list += [ma.comp_max_imgs(dir_in + condition, base_name, dir_maxProj_raw + condition)]
    endTime = datetime.now() - startTime
    print(endTime)
else:
    # Read maximum projection images (runtime ~= 20 s )
    print('Reading maximum projection images')
    startTime = datetime.now()
    max_img_list = []
    for condition in conditions:
        max_img_list += [ma.read_max_imgs(dir_maxProj_raw + condition, base_name)]
    endTime = datetime.now() - startTime
    print(endTime)

#%% Plot histograms of raw images (approx. 29s per condition)
for condition, max_img_list_condition in zip(conditions, max_img_list):
    ma.plotHist_perChannel_imgset_list(max_img_list_condition, dir_in + condition, dir_results_intinspect + condition, name_tag = 'original')

#%% Measure the spread of intensities in each channel
"""The spread of intensities is quantified by the standard deviation of the 
intensities that are over a threshold. The threshold value is set manually 
and it defines the minimum intensity associated with the presence of protein.
Image regions with intensity values under the threshold are considered to 
image air, i.e. absence of protein."""

# Protein presence/absence (air) intensity threshold
th_int = 15 #could it be set automatically based on the data?

# Compute stds per image channel and, for each sample, plot intensity histograms per channel.
std_list = []
for condition, max_img_list_condition in zip(conditions, max_img_list):
    std_list += [ma.comp_std_perChannel(max_img_list_condition, th_int, dir_in + condition)] 

# Histogram of standard deviations (std) per image channel 
fig, axs = plt.subplots(len(conditions),4, figsize = (4*2,len(conditions)*2), sharex = True, sharey = True)
plt.suptitle('Distribution of intensity spread across images\n Rows: ' +  ', '.join(conditions))
for ind_c,std_list_condition in enumerate(std_list):  
    for ind,channel_list in enumerate(std_list_condition):
        axs[0][ind+1].title.set_text('Channel ' + str(ind+1))
        axs[ind_c][ind+1].hist(channel_list, bins = 50, density = True)
  
# Histogram of standard deviations per image  
axs[0][0].title.set_text('All channels')
std_list_flat = []
for ind_c,std_list_condition in enumerate(std_list):
    std_list_condition_flat = ma.flatten_list(std_list_condition) 
    axs[ind_c][0].hist(std_list_condition_flat, bins = 50, density = True)    
    std_list_flat += std_list_condition_flat
plt.show()    
    
# Average intesity-spread (std) across all image channels
mean_std = sum(std_list_flat)/len(std_list_flat)
print('The average spread of an image channel is ' + str(round(mean_std)))
 
#%% Correct intensities (34s per condition)
max_img_corr_list = []
for condition, max_img_list_condition in zip(conditions, max_img_list):
    max_img_corr_list += [ma.intensity_spread_normalisation(copy.deepcopy(max_img_list_condition), th_int, mean_std, dir_in + condition, base_name, dir_maxProj_processed + condition)]

#%% Plot histograms of corrected images (29s per condition)
for condition, max_img_corr_list_condition in zip(conditions, max_img_corr_list):
   ma.plotHist_perChannel_imgset_list(max_img_corr_list_condition, dir_in + condition, dir_results_intinspect + condition, name_tag = 'corrected')

#%% Visualise raw vs. corrected images
for max_img_list_condition, max_img_corr_list_condition, condition in zip(max_img_list, max_img_corr_list, conditions):
    ma.compare_imgpairs_list(max_img_list_condition, max_img_corr_list_condition , dir_in + condition, dir_results_intinspect + condition)  

 #%% Exchange channels: fibrillar collagen with elastin
for max_img_list_condition in max_img_corr_list:
    print(len(max_img_list_condition))
    for max_img_list_sample in max_img_list_condition:
        print(len(max_img_list_sample))
        for nr_frame,max_img in enumerate(max_img_list_sample):
            max_img_list_sample[nr_frame]= max_img[:,:,[0,2,1]]
            
#%% Save images in cmy colourspace
for max_img_corr_list_condition, condition in zip(max_img_corr_list, conditions):
    ma.rgb2cmy_list(max_img_corr_list_condition, dir_in + condition, base_name, dir_maxProj_processed_cmy + condition)  

#%%% Print total computational time
end_time_global = datetime.now() - startTime
print(end_time_global)