#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Explorative analysis of protein composition and structure in lung ECM to 
study disease-induced remodeling of the three proteins that provide structural 
integrity to the lung: collagen IV, elastin and fibrillar collagen.

The three-protein analysis provides the following:
    - A 'Data Model', a dictionary of image patches representative of the local 
    protein structure and composition found in the data (both healthy and 
    disease group) at the scale defined by the patch size.
    - A 'Sorted Data Model' or disease signature, a sorted version of the 
    dictionary above, based on how often a dictionary element (called 
    cluster centre in the publication) is found in the healthy vs. the disease.
    - A 'Probability Map' for every image based on the three proteins, with 
    pixel correspondance to the input images (maximum projections), 
    highlighting regions characteristic of health or disease. 
    - A 'Disease Effect', to quantify how well the control and 
    disease groups are separated after assigning a health probability score to 
    each image, computed as the average of its probability map values.
    
by Monica J. Emerson monj@dtu.dk and Anders B. Dahl abda@dtu.dk

For more information, read our article (add link and citation)"""

#%% Load packages and initialise parameters

import numpy as np
import matplotlib.pyplot as plt
import sklearn.cluster
import microscopy_analysis as ma
import skimage.io 
import skimage.transform
 
import os
from datetime import datetime
import sys
from matplotlib.offsetbox import OffsetImage, AnnotationBbox
from math import ceil

startTime = datetime.now()

plt.close('all')

sc_fac = 0.5 #25 #25 #0.5 # scaling factor of the image
patch_size = 17
nr_clusters = 100 #49#100 # number of clusters, choose a value with an integer as a square root
protein_mode = 'triple'
colour_mode = 'cmy' #'bnw' #colour
th_back = 15
nr_runs = 10 #5
 
#%% Define and create directories

disease = 'covid' #diseased (mix, 2 of each condition)' #''emphysema' 'sarcoidosis' 'ild' 'covid'
control = 'control'
conditions = [disease, control]
    
# input directories - images start with the name 'frame'
base_dir_in = '../input_data/'
#dir_in = base_dir_in + 'maximum_projections' #uncomment to read maximum projections without contrast equalisation
dir_in = base_dir_in + 'postprocessed_maxProjs/rgb/' #Read rgb for correct protein channel ordering, visualisation will be in cmy
base_name = 'frame'

#output directory
dir_results = '../results/' #rerun just to make sure not to muck up the results
os.makedirs(dir_results, exist_ok = True)  
os.makedirs(dir_results + '/statistics', exist_ok = True)  

#%% Read (preprocessed) maximum corrected images
#Read maximum projection images (runtime ~= 20 s )
print('Reading maximum projection images')

max_img_list_control = ma.read_max_imgs(dir_in + control, base_name)
max_img_list_disease = ma.read_max_imgs(dir_in + disease, base_name)

#%% Exchange channels: fibrillar collagen with elastin
for max_img_sample_control in max_img_list_control:
    for nr_frame, max_img_control in enumerate(max_img_sample_control):
        max_img_sample_control[nr_frame] = max_img_control[:,:,[0,2,1]]

for max_img_sample_disease in max_img_list_disease:
    for nr_frame, max_img_disease in enumerate(max_img_sample_disease):
        max_img_sample_disease[nr_frame] = max_img_disease[:,:,[0,2,1]]

#%% Prepare maximum projection images for feature extraction
#Flatten and merge conditions
max_img_list_control_flat = ma.flatten_list(max_img_list_control) 
max_img_list_disease_flat = ma.flatten_list(max_img_list_disease)

#Rescale
max_img_list_control_processed = []
for max_img in ma.flatten_list(max_img_list_control) :
    max_img_list_control_processed += [skimage.transform.rescale(max_img, sc_fac, preserve_range = True, multichannel=True).astype(max_img.dtype)]

max_img_list_disease_processed = []
for max_img in ma.flatten_list(max_img_list_disease) :
    max_img_list_disease_processed += [skimage.transform.rescale(max_img, sc_fac, preserve_range = True, multichannel=True).astype(max_img.dtype)]

#%% Compute patches
patch_feat_list_control = []
for max_img in max_img_list_control_processed:
    patch_feat_list_control += [ma.ndim2col_pad(max_img, (patch_size, patch_size),norm=False).transpose()]

patch_feat_list_disease = []
for max_img in max_img_list_disease_processed:
    patch_feat_list_disease += [ma.ndim2col_pad(max_img, (patch_size, patch_size),norm=False).transpose()]

patch_feat_total = patch_feat_list_control + patch_feat_list_disease
    
for run in range(nr_runs):
    
    #directories for results from the different runs
    dir_probs = dir_results + disease + '_' + protein_mode + '_%dclusters_%ddownscale_%dpatchsize/'%(nr_clusters,1/sc_fac,patch_size)
    dir_probs += 'run'+str(run)+'/'
    ma.make_output_dirs(dir_probs, conditions)
    dir_probs_noBack = dir_probs + 'noBackground/'
    os.makedirs(dir_probs_noBack, exist_ok = True) 
    
    #%% k-means clustering
    startTime_clustering = datetime.now()
    
    batch_size = 10000
    th_nr_pathesINcluster = 20
    th_nr_weakClusters = 0 #added for covid and ild
    
    #If cluster centers were already computed, use those
    if os.path.exists(dir_probs + 'array_cluster_centers_colour.npy'):
         cluster_centers = np.load(dir_probs + 'array_cluster_centers_colour.npy')
         kmeans = sklearn.cluster.MiniBatchKMeans(n_clusters=nr_clusters, batch_size = batch_size)
         kmeans.cluster_centers_ = cluster_centers
    
    #Compute cluster centres
    else: 
        #features for clustering
        nr_keep = 10000 # number of features/image randomly picked for clustering 
        n_im = len(patch_feat_total)
        feat_dim = patch_feat_total[0].shape[1]
        patch_feat_to_cluster = np.zeros((nr_keep*n_im,feat_dim),dtype = max_img.dtype)
        
        f = 0
        nr_keep_control = nr_keep//(2*len(patch_feat_list_control)//len(patch_feat_list_disease))
        nr_keep_disease = nr_keep//2
        for patch_feat in patch_feat_list_control:
            keep_indices = np.random.permutation(np.arange(patch_feat.shape[0]))[:nr_keep_control]
            patch_feat_to_cluster[f:f+nr_keep_control,:] = patch_feat[keep_indices,:]
            f += nr_keep
        
        for patch_feat in patch_feat_list_disease:
            keep_indices = np.random.permutation(np.arange(patch_feat.shape[0]))[:nr_keep_disease]
            patch_feat_to_cluster[f:f+nr_keep_disease,:] = patch_feat[keep_indices,:]
            f += nr_keep
            
        #Get feature vectors
        feat_to_cluster = patch_feat_to_cluster
        
        #Compute cluster centers and save if there is no weak clusters
        nr_weakClusters = th_nr_weakClusters+1
        repetition = 0
        nr_weakClusters_list = []
        while nr_weakClusters>th_nr_weakClusters: #While weak clusters is different from empty recompute
            kmeans = sklearn.cluster.MiniBatchKMeans(n_clusters=nr_clusters, batch_size = batch_size, verbose = False, reassignment_ratio=0.0005, max_no_improvement = 100) #0.00075 was good for uip
            kmeans.fit(feat_to_cluster)
            
            #Nr. patches that have contributed to each cluster
            nr_feat_in_cluster = [sum(kmeans.labels_==cluster) for cluster in range(nr_clusters)]
            
            weak_cluster_IDs = [ind for ind,val in enumerate(nr_feat_in_cluster) if val<th_nr_pathesINcluster]
            print('Weak cluster IDs: ',weak_cluster_IDs)
                
            #Weak clusters are those composed by very few patches
            nr_weakClusters = len([1 for i in nr_feat_in_cluster if i<th_nr_pathesINcluster])
            
            #If there are weak clusters, the clustering should be recomputed
            if nr_weakClusters>th_nr_weakClusters:
                print(str(nr_weakClusters)+ " clusters composed of less than "+str(th_nr_pathesINcluster)+" images") 
            else:
                np.save(dir_probs + 'array_cluster_centers_colour.npy', kmeans.cluster_centers_)    # .npy extension is added if not given
                
            print(repetition)
            repetition += 1
            nr_weakClusters_list += [nr_weakClusters]
            
     
    endTime_clustering = datetime.now() - startTime_clustering
    print(endTime_clustering)
    
    #%% Plot all cluster centers (grid view)
    cluster_centers = kmeans.cluster_centers_
    int_sum_list = ma.plot_grid_cluster_centers(cluster_centers, range(nr_clusters), patch_size, colour_mode = colour_mode)
    plt.savefig(dir_probs + 'clustercenters_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #%% Histograms for background, control and disease
    hist_control, assignment_list_control= ma.compute_assignment_hist(patch_feat_list_control, kmeans)
    hist_disease, assignment_list_disease = ma.compute_assignment_hist(patch_feat_list_disease, kmeans)
    
    #%% Show histograms (bar plot) of control and disease
    fig, ax = plt.subplots(1,1)
    ax.bar(np.array(range(0,nr_clusters))-0.25, hist_control, width = 0.5, label='Control', color = 'b')
    ax.bar(np.array(range(0,nr_clusters))+0.25, hist_disease, width = 0.5, label='Disease', color = 'r')
    ax.legend()
    plt.savefig(dir_probs + 'assignmentHistograms_%dclusters_%ddownscale_%dpatchsize.pdf'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    
    #%% Compute populated, occurrence and importance cluster measures from the histogram

    #OCCURRENCE RATIO: How more often does it occur in one of the conditions
    occurrence_ratio = np.empty((nr_clusters))
    ind_control = hist_control>hist_disease
    ind_disease = ~ind_control
    control_cond = (hist_disease!=0) & ind_control
    occurrence_ratio[control_cond] = hist_control[control_cond]/hist_disease[control_cond]
    disease_cond = (hist_control!=0) & ind_disease
    occurrence_ratio[disease_cond] = -hist_disease[disease_cond]/hist_control[disease_cond]
    
    #POPULATED: How trustworthy is the cluster? 
    populated = hist_control+hist_disease 
    weights = populated/populated.max()
    
    #Cluster IMPORTANCE: occurrence weighed by POPULATED
    def sigmoid(x,k):
        return 2 / (1 + np.exp(-k*x)) - 1
    
    x = np.linspace(0,1,100)
    plt.plot(x,sigmoid(x,100))
    
    importance = occurrence_ratio * sigmoid(weights,100)
    
    #%% Plot all control and disease cluster centers in the 2d populated/occurrence space
    fig_control, ax_control = plt.subplots(figsize=(15,15))
    ax_control.scatter(populated[occurrence_ratio>1], occurrence_ratio[occurrence_ratio>1]) 
    ax_control.set_ylim(0.8,7)
    ax_control.set_xlim(0,0.08)
    
    fig_disease, ax_disease = plt.subplots(figsize=(15,15))
    ax_disease.scatter(populated[occurrence_ratio<1], -occurrence_ratio[occurrence_ratio<1]) 
    ax_disease.set_ylim(0.8,7)
    ax_disease.set_xlim(0,0.08)
    
    for x0, y0, cluster_nr in zip(populated, occurrence_ratio, np.arange(0,nr_clusters)):
        cluster_centre = np.transpose((np.reshape(kmeans.cluster_centers_[cluster_nr,:],(3,patch_size,patch_size))),(1,2,0))
        cluster_centre = ma.rgb2cmy(cluster_centre)
        if y0>0:
            ab = AnnotationBbox(OffsetImage(cluster_centre.astype(max_img.dtype)), (x0, y0), frameon=False)
            ax_control.add_artist(ab)
        else:
            ab = AnnotationBbox(OffsetImage(cluster_centre.astype(max_img.dtype)), (x0, -y0), frameon=False)
            ax_disease.add_artist(ab)
    
    plt.figure(fig_control.number)
    plt.savefig(dir_probs + 'controlClustercenters_2Dspace_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
        
    plt.figure(fig_disease.number)
    plt.savefig(dir_probs + 'diseaseClustercenters_2Dspace_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
       
    #%% Plot all clusters sorted by IMPORTANCE
    
    #According to occurence
    clusters_control_occ_sorted = [np.arange(0,100)[i] for i in np.argsort(occurrence_ratio.flatten()) if (occurrence_ratio.flatten())[i]>0]
    clusters_control_occ_sorted.reverse()
    
    clusters_disease_occ_sorted = [np.arange(0,100)[i] for i in np.argsort(occurrence_ratio.flatten()) if (occurrence_ratio.flatten())[i]<0]
    
    clusters_occ_sorted = [np.arange(0,100)[i] for i in np.argsort(occurrence_ratio.flatten())]
    clusters_occ_sorted.reverse()
    
    occurrence_ratio_sorted = [occurrence_ratio[i] for i in np.argsort(occurrence_ratio.flatten())]
    occurrence_ratio_sorted.reverse()
    
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_occ_sorted, patch_size, colour_mode = colour_mode)
    plt.suptitle('Cluster centers \n sorted by occ from control to disease with cluster ' + str(len(clusters_control_occ_sorted))  + ' in between')
    plt.savefig(dir_probs + 'clustercenters_occ_sorted_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #According to occurence weighed by populated
    clusters_control_occW_sorted = [np.arange(0,100)[i] for i in np.argsort(importance.flatten()) if (importance.flatten())[i]>0]
    clusters_control_occW_sorted.reverse()
    
    clusters_disease_occW_sorted = [np.arange(0,100)[i] for i in np.argsort(importance.flatten()) if (importance.flatten())[i]<0]
    
    clusters_occW_sorted = [np.arange(0,100)[i] for i in np.argsort(importance.flatten())]
    clusters_occW_sorted.reverse()
    
    occurrence_ratio_sorted = [occurrence_ratio[i] for i in np.argsort(importance.flatten())]
    occurrence_ratio_sorted.reverse()
    
    cluster_nums = np.arange(0,nr_clusters)
    cluster_nums_sorted = [cluster_nums[i] for i in np.argsort(importance.flatten())]
    cluster_nums_sorted.reverse()
    
    prob_control = hist_control/(hist_control+hist_disease)
    prob_control_sorted = [prob_control[i] for i in np.argsort(importance.flatten())]
    prob_control_sorted.reverse()
    
    
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_occW_sorted, patch_size,  colour_mode = colour_mode)
    plt.suptitle('Cluster centers \n sorted by occ*f(p) from control to disease with cluster ' + str(len(clusters_control_occW_sorted))  + ' in between')
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #with probabilities
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_occW_sorted, patch_size,  colour_mode = colour_mode, titles = True, occurrence = prob_control_sorted)
    plt.suptitle('Cluster centers \n sorted by occ*f(p) from control to disease with cluster ' + str(len(clusters_control_occW_sorted))  + ' in between')
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_wProbs_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #with cluster numbers
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_occW_sorted, patch_size,  colour_mode = colour_mode, titles = True, occurrence = cluster_nums_sorted)
    plt.suptitle('Cluster centers \n sorted by occ*f(p) from control to disease with cluster ' + str(len(clusters_control_occW_sorted))  + ' in between')
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_wClusternums_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #%% Plot all clusters sorted - split channelwise view
    ma.plot_grid_cluster_centers(np.copy(kmeans.cluster_centers_), clusters_occW_sorted, patch_size, protein_mode = protein_mode, colour_mode = 'cmy', channel = 0)
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_split_channel1_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    ma.plot_grid_cluster_centers(np.copy(kmeans.cluster_centers_), clusters_occW_sorted, patch_size, protein_mode = protein_mode, colour_mode = 'cmy', channel = 1)
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_split_channel2_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    ma.plot_grid_cluster_centers(np.copy(kmeans.cluster_centers_), clusters_occW_sorted, patch_size, protein_mode = protein_mode, colour_mode = 'cmy', channel = 2)
    plt.savefig(dir_probs + 'clustercenters_occW_sorted_split_channel3_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)

    
    #%% Find background clusters
    clusters_background_intBased = [i for i in range(len(int_sum_list)) if int_sum_list[i] < th_back*patch_size**2]

    clusters_background = clusters_background_intBased
    print('Background clusters'+str(clusters_background))
    
    #%% Plot grid of ordered, non-background clusters that are populated over a thresh
    th_populated = 0.5/nr_clusters
    th_occurr = 1.25
    
    #control
    clusters_control_occW_sorted_noBackground = [i for i in clusters_control_occW_sorted if (i not in clusters_background)&(populated[i]>th_populated)&(occurrence_ratio[i]>th_occurr)]
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_control_occW_sorted_noBackground, patch_size, colour_mode = colour_mode)
    #plt.suptitle('Characteristic clusters for control, excluding background, \n from most important to least')
    plt.savefig(dir_probs_noBack + 'clustercenters_control_occW_sorted_noBack%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #disease
    clusters_disease_occW_sorted_noBackground = [i for i in clusters_disease_occW_sorted if (i not in clusters_background)&(populated[i]>th_populated)&(abs(occurrence_ratio[i])>th_occurr)]
    ma.plot_grid_cluster_centers(kmeans.cluster_centers_, clusters_disease_occW_sorted_noBackground, patch_size, colour_mode = colour_mode)
    #plt.suptitle('Charactertistic clusters for ' + disease + ', excluding background, \n from most important to least')
    plt.savefig(dir_probs_noBack + 'clustercenters_disease_occW_sorted_noBack%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
     
    #%% Plot channelwise split characteristic clusters 
    nr_clusters_display = 10
    clusters_control = clusters_control_occW_sorted_noBackground[:nr_clusters_display] 
    clusters_disease = clusters_disease_occW_sorted_noBackground[:nr_clusters_display] 
    
    #control clusters 
    cluster_centers_control = kmeans.cluster_centers_[clusters_control]
    fig, axs = plt.subplots(1,len(clusters_control), figsize=(len(clusters_control)*3,3), sharex=True, sharey=True)
    fig.suptitle('Cluster centers for control')
 
    fig_split, axs_split = plt.subplots(3,len(clusters_control), figsize=(len(clusters_control)*3,9), sharex=True, sharey=True)
    fig_split.suptitle('Cluster centers for control - split')
    for l in np.arange(0,len(clusters_control)):
   
        cluster_centre = np.transpose((np.reshape(cluster_centers_control[l,:],(3,patch_size,patch_size))),(1,2,0))
    
        #channel 1 - collagen IV
        im = np.zeros(cluster_centre.shape)
        im[...,0] = cluster_centre[...,0]
        im = ma.rgb2cmy(im)
        axs_split[0][l].imshow(im.astype(np.uint8))
        axs_split[0][l].axis('off')
        axs_split[0][l].set_title(clusters_control[l])
         
        #channel 2 - elastin
        im = np.zeros(cluster_centre.shape)
        im[...,1] = cluster_centre[...,1]
        im = ma.rgb2cmy(im)
        axs_split[1][l].imshow(im.astype(np.uint8))
        axs_split[1][l].axis('off')
         
        #channel 3 - SHG
        im = np.zeros(cluster_centre.shape)
        im[...,2] = cluster_centre[...,2]
        im = ma.rgb2cmy(im)
        axs_split[2][l].imshow(im.astype(np.uint8))
        axs_split[2][l].axis('off')
        
        plt.savefig(dir_probs_noBack + 'clustercenters_control_split_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
            
        #cmy merge
        cluster_centre = ma.rgb2cmy(cluster_centre)
        axs[l].imshow(cluster_centre.astype(np.uint8))
        axs[l].axis('off')
        axs[l].set_title(clusters_control[l])
        
        plt.figure(fig.number),
        plt.savefig(dir_probs_noBack + 'clustercenters_control_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
        
    #disease clusters 
    cluster_centers_disease = kmeans.cluster_centers_[clusters_disease]
    fig, axs = plt.subplots(1,len(clusters_disease), figsize=(len(clusters_disease)*3,3), sharex=True, sharey=True)
    fig.suptitle('Cluster centers for disease')
 
    fig_split, axs_split = plt.subplots(3,len(clusters_disease), figsize=(len(clusters_disease)*3,9), sharex=True, sharey=True)
    fig_split.suptitle('Cluster centers for disease - split')
    for l in np.arange(0,len(clusters_disease)):
   
        cluster_centre = np.transpose((np.reshape(cluster_centers_disease[l,:],(3,patch_size,patch_size))),(1,2,0))
    
        #channel 1 - collagen IV
        im = np.zeros(cluster_centre.shape)
        im[...,0] = cluster_centre[...,0]
        im = ma.rgb2cmy(im)
        axs_split[0][l].imshow(im.astype(np.uint8))
        axs_split[0][l].axis('off')
        axs_split[0][l].set_title(clusters_disease[l])
         
        #channel 2 - elastin
        im = np.zeros(cluster_centre.shape)
        im[...,1] = cluster_centre[...,1]
        im = ma.rgb2cmy(im)
        axs_split[1][l].imshow(im.astype(np.uint8))
        axs_split[1][l].axis('off')
         
        #channel 3 - SHG
        im = np.zeros(cluster_centre.shape)
        im[...,2] = cluster_centre[...,2]
        im = ma.rgb2cmy(im)
        axs_split[2][l].imshow(im.astype(np.uint8))
        axs_split[2][l].axis('off')
        
        plt.savefig(dir_probs_noBack + 'clustercenters_disease_split_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
            
        #cmy merge
        cluster_centre = ma.rgb2cmy(cluster_centre)
        axs[l].imshow(cluster_centre.astype(np.uint8))
        axs[l].axis('off')
        axs[l].set_title(clusters_disease[l])
        
        plt.figure(fig.number),
        plt.savefig(dir_probs_noBack + 'clustercenters_disease_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
        
    #%% Compute probability images                
    startTime_probs = datetime.now()

    #Assign probabilities of control and disease to clusters
    prob_control= hist_control/(hist_control+hist_disease)
    prob_disease = hist_disease/(hist_control+hist_disease)
    
    #Compute probability images
    r,c,_ = max_img_list_control_flat[0].shape
    img_shape = (int(r*sc_fac),int(c*sc_fac))
    prob_img_list_control = ma.comp_prob_imgs(prob_control,assignment_list_control,img_shape)
    prob_img_list_disease = ma.comp_prob_imgs(prob_control,assignment_list_disease,img_shape) 
  
    #%% Display probability images and accumulate probs per frame and sample
    nr_frames = len(max_img_list_control[0])
    
    #control
    sample_names_control = ma.listdir_custom(dir_in + control, string_start_pos = -4, dir_flag = True, )
    
    nr_list = 0
    prob_list_control= []
    fig_probmaps, ax_probmaps = plt.subplots(nr_frames, len(sample_names_control),figsize = (len(sample_names_control),nr_frames), sharex=True, sharey=True)
    for max_img_list_sample,sample_name,sample_nr in zip(max_img_list_control, sample_names_control, range(len(sample_names_control))): #in general,  in max_img_list_condition
        print(sample_name)
        nr_frames = len(max_img_list_sample)
        prob_img_list_sample = prob_img_list_control[nr_list:nr_list+nr_frames]
        nr_list += nr_frames
        
        prob_sample = np.zeros((nr_frames,1))
        fig, axs = plt.subplots(2,nr_frames, figsize = (nr_frames*2,2*2), sharex=True, sharey=True)
        for prob_img, max_img, frame_nr in zip(prob_img_list_sample,max_img_list_sample,range(nr_frames)):
            max_img_ds = skimage.transform.rescale(max_img, sc_fac, multichannel = True, preserve_range = True).astype(max_img.dtype)
            
            #compute image probability
            prob_frame = sum(sum(prob_img))/prob_img.size
            
            #remove background from prob imgs
            ind_back = (max_img_ds[:,:,0]<th_back)&(max_img_ds[:,:,1]<th_back)&(max_img_ds[:,:,2]<th_back)
            prob_img[ind_back] = 0.5
            
            #plot
            ax_probmaps[frame_nr][sample_nr].imshow(1-prob_img, cmap=plt.cm.bwr, vmin = 0, vmax = 1)
            ax_probmaps[frame_nr][sample_nr].xaxis.set_ticklabels([])
            ax_probmaps[frame_nr][sample_nr].xaxis.set_ticks_position('none')
            ax_probmaps[frame_nr][sample_nr].yaxis.set_ticklabels([])
            ax_probmaps[frame_nr][sample_nr].yaxis.set_ticks_position('none')
             
            axs[1][frame_nr].imshow(1-prob_img, cmap=plt.cm.bwr, vmin = 0, vmax = 1)
            axs[1][frame_nr].set_title("%.2f" % np.round(prob_frame,2))
            axs[1][frame_nr].xaxis.set_ticklabels([])
            axs[1][frame_nr].xaxis.set_ticks_position('none')
            axs[1][frame_nr].yaxis.set_ticklabels([])
            axs[1][frame_nr].yaxis.set_ticks_position('none')
             
            if colour_mode == 'cmy':
                 max_img_ds = ma.rgb2cmy(max_img_ds)
            axs[0][frame_nr].imshow(max_img_ds)
            axs[0][frame_nr].xaxis.set_ticklabels([])
            axs[0][frame_nr].xaxis.set_ticks_position('none')
            axs[0][frame_nr].yaxis.set_ticklabels([])
            axs[0][frame_nr].yaxis.set_ticks_position('none')
            
            #save img probs
            prob_sample[frame_nr] = prob_frame 
        prob_list_control += [prob_sample]
        
        plt.suptitle('Health probability of sample '+ sample_name +', avg:' + str(round(prob_sample.mean(),2))+ ' std:' + str(round(prob_sample.std(),2)))
        plt.savefig(dir_probs + '/' + control + '/' + 'probImHealth_'+sample_name+'_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
        #plt.show() 
    plt.figure(fig_probmaps.number),plt.savefig(dir_probs + '/' + control + '/' + 'probMaps_'+control+'_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    #plt.show()
        
    #disease
    sample_names_disease = ma.listdir_custom(dir_in + disease, string_start_pos = -4, dir_flag = True)
    
    nr_list = 0
    prob_list_disease = []
    fig_probmaps, ax_probmaps = plt.subplots(nr_frames, len(sample_names_disease),figsize = (len(sample_names_disease),nr_frames), sharex=True, sharey=True)
    for max_img_list_sample,sample_name,sample_nr in zip(max_img_list_disease, sample_names_disease, range(len(sample_names_disease))): #in general,  in max_img_list_condition
        print(sample_name)
        nr_frames = len(max_img_list_sample)
        prob_img_list_sample = prob_img_list_disease[nr_list:nr_list+nr_frames]
        nr_list += nr_frames
        
        prob_sample = np.zeros((nr_frames,1))
        fig, axs = plt.subplots(2,nr_frames, figsize = (nr_frames*2,2*2), sharex=True, sharey=True)
        for prob_img, max_img, frame_nr in zip(prob_img_list_sample,max_img_list_sample,range(nr_frames)):
             max_img_ds = skimage.transform.rescale(max_img, sc_fac, multichannel = True, preserve_range = True).astype(max_img.dtype)
            
             #compute image probability
             prob_frame = sum(sum(prob_img))/prob_img.size
            
             #remove background from prob imgs
             ind_back = (max_img_ds[:,:,0]<th_back)&(max_img_ds[:,:,1]<th_back)&(max_img_ds[:,:,2]<th_back)
             prob_img[ind_back] = 0.5
            
             #plot
             ax_probmaps[frame_nr][sample_nr].imshow(1-prob_img, cmap=plt.cm.bwr, vmin = 0, vmax = 1)
             ax_probmaps[frame_nr][sample_nr].xaxis.set_ticklabels([])
             ax_probmaps[frame_nr][sample_nr].xaxis.set_ticks_position('none')
             ax_probmaps[frame_nr][sample_nr].yaxis.set_ticklabels([])
             ax_probmaps[frame_nr][sample_nr].yaxis.set_ticks_position('none')
             
             axs[1][frame_nr].imshow(1-prob_img, cmap=plt.cm.bwr, vmin = 0, vmax = 1)
             axs[1][frame_nr].set_title("%.2f" % np.round(prob_frame,2))
             axs[1][frame_nr].xaxis.set_ticklabels([])
             axs[1][frame_nr].xaxis.set_ticks_position('none')
             axs[1][frame_nr].yaxis.set_ticklabels([])
             axs[1][frame_nr].yaxis.set_ticks_position('none')
             
             if colour_mode == 'cmy':
                  max_img_ds = ma.rgb2cmy(max_img_ds)
             axs[0][frame_nr].imshow(max_img_ds)
             axs[0][frame_nr].xaxis.set_ticklabels([])
             axs[0][frame_nr].xaxis.set_ticks_position('none')
             axs[0][frame_nr].yaxis.set_ticklabels([])
             axs[0][frame_nr].yaxis.set_ticks_position('none')
            
             #save img probs
             prob_sample[frame_nr] = prob_frame 
        prob_list_disease += [prob_sample]
        
        plt.suptitle('Health probability of sample '+ sample_name +', avg:' + str(round(prob_sample.mean(),2))+ ' std:' + str(round(prob_sample.std(),2)))
        plt.savefig(dir_probs + '/' + disease + '/' + 'probImHealth_'+sample_name+'_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
        #plt.show() 
    plt.figure(fig_probmaps.number),plt.savefig(dir_probs + '/' + disease + '/' + 'probMaps_'+disease+'_%dclusters_%ddownscale_%dpatchsize.png'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    #plt.show()
    
    prob_list_samples = prob_list_control + prob_list_disease   
         
    #%% Box plots
    fig, ax = plt.subplots(figsize = (15,5))
    ax.set_title('Image health scores ')
    
    sample_sep = 0.85
    patient_sep = 2.35
    condition_sep = 2.5 
    positions_control = np.tile([0,0+sample_sep],int(len(max_img_list_control)/2)) + np.repeat(np.arange(0,len(max_img_list_control)/2*patient_sep,patient_sep),2)
    positions_disease = np.tile([len(max_img_list_control)+condition_sep,len(max_img_list_control)+condition_sep+sample_sep],int(len(max_img_list_disease)/2)) + np.repeat(np.arange(0,len(max_img_list_disease)/2*patient_sep,patient_sep),2)
    array_probs = np.squeeze(np.asarray(prob_list_samples))
    bp_control = ax.boxplot((array_probs[:len(max_img_list_control),...]).T, positions = positions_control, patch_artist = True)
    bp_disease = ax.boxplot(array_probs[len(max_img_list_control):,...].T, positions = positions_disease, patch_artist = True)
    
    for element in ['boxes', 'whiskers', 'fliers', 'means', 'medians', 'caps']:
            plt.setp(bp_control[element], color='blue')
            plt.setp(bp_control[element], linewidth = 2)
            
    for patch in bp_control['boxes']:
            patch.set(facecolor='white')
            patch.set(linewidth = 2)
            
    for element in ['boxes','whiskers', 'fliers', 'means', 'medians', 'caps']:
            plt.setp(bp_disease[element], color='red')
            plt.setp(bp_disease[element], linewidth = 2)
            
    for patch in bp_disease['boxes']:
            patch.set(facecolor= 'white')  
            patch.set(linewidth = 2) 
        #patch.set(alpha=0.2) 
        
    ax.set_xticklabels([i[-4:] for i in sample_names_control]+[i[-4:] for i in sample_names_disease])
    ax.set_ylim(0.3,0.7)
    ax.set_ylabel('Probability health')
    #ax.set_ylim(0,1)
    
    ax.tick_params(
        axis='both',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        #labelleft = False,      # ticks along the top edge are off
        #labelbottom=False
        )
    
    #plot points
    positions = list(positions_control) + list(positions_disease)
    for prob_sample, position in zip(prob_list_samples, positions):
        print(prob_sample)
        print(5*[position])
        # Add some random "jitter" to the x-axis
        x = np.random.normal(5*[position], 0)
        #plot(x, y, 'r.', alpha=0.2)
        ax.plot(x, prob_sample,'k.', markersize = 6, zorder = 3)
       
    #horizontal line
    x = np.linspace(0,position.max())    
    y = 0.5*np.ones(x.shape)
    ax.plot(x,y,'k',linewidth = 1, linestyle = 'dashed')
    
    plt.savefig(dir_probs + 'scatterBoxplots_%dclusters_%ddownscale_%dpatchsize.pdf'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    
    fig2, ax2 = plt.subplots(figsize = (15,5))
    #plot points
    for prob_sample, position in zip(prob_list_control, positions_control):
        ax2.plot(5*[position], prob_sample,'b.', markersize = 4)
        
    for prob_sample, position in zip(prob_list_disease, positions_disease):
        ax2.plot(5*[position], prob_sample,'r.', markersize = 4)
    
    #horizontal line
    x = np.linspace(0,position.max())    
    y = 0.5*np.ones(x.shape)
    ax2.plot(x,y,'k',linewidth = 1, linestyle = 'dashed')
    
    ax2.set_xticks(ax.get_xticks())
    ax2.set_xticklabels([i[-4:] for i in sample_names_control]+[i[-4:] for i in sample_names_disease])
    ax2.set_ylim(0.3,0.7)
    ax2.set_ylabel('Probability health')

    plt.savefig(dir_probs + 'samplewiseScatterPlots_%dclusters_%ddownscale_%dpatchsize.pdf'%(nr_clusters,1/sc_fac,patch_size), dpi=300)
    
       
    #%% Print computational time
    endTime = datetime.now() - startTime
    print(endTime)
      
    
    #%%ANOVA SETUP
    os.makedirs(dir_results + 'statistics/', exist_ok = True)  
    
    from statsmodels.formula.api import ols
    from statsmodels.stats.anova import anova_lm
    import pandas as pd
    from patsy import ModelDesc
    
    nr_frames_patient = 10
    img_nr = np.tile(np.arange(nr_frames),array_probs.size//nr_frames)
    sample_nr = np.tile(np.repeat(np.arange(nr_frames_patient//nr_frames),nr_frames),array_probs.size//nr_frames_patient)
    #patient_nr = np.tile(np.repeat(np.arange(array_probs.size//nr_frames_patient//len(conditions)),nr_frames_patient),len(conditions))
    patient_nr = np.concatenate([np.repeat(np.arange(len(max_img_list_control_flat)//nr_frames_patient),nr_frames_patient),np.repeat(np.arange(len(max_img_list_disease_flat)//nr_frames_patient),nr_frames_patient)])
    condition_nr = np.concatenate([np.zeros(len(max_img_list_control_flat)), np.ones(len(max_img_list_disease_flat))])
    
    
    dep_var = array_probs.flatten()
        
    data_dict = {'Image':img_nr,
                 'Sample':sample_nr,
                 'Patient':patient_nr,
                 'Condition': condition_nr,
                 'S':dep_var}
    
    formula = 'S ~ C(Condition)/C(Patient)/C(Sample)'
    
    desc = ModelDesc.from_formula(formula)
    print(desc.describe())
        
    lm = ols(formula, data_dict).fit()
    print(lm.summary())
    
    anova_table = anova_lm(lm)
    print(anova_table)
    writer_anova = pd.ExcelWriter(dir_results + 'statistics/statistics_anova_' + disease + '_' + protein_mode + '_ds'+ str(1/sc_fac) +'_run' + str(run) +'.xlsx', engine='xlsxwriter')
    anova_table.to_excel(writer_anova,sheet_name = disease)
    writer_anova.save()
    
    





